@@ -37,6 +37,7 @@
 variable.fct	<- function(			# function that creates starting and end period when deriving instruments from data for endogenous variables
   varname
   ,i
+  ,inst.collapse
   ,T.mcDiff
   ,lagTerms
   #   ,mc.ref.t
@@ -49,12 +50,22 @@ variable.fct	<- function(			# function that creates starting and end period when
 
   ti.temp   <- rep(1, times = Time-lagTerms-1) + if(Time-lagTerms-1 - T.mcDiff > 0){c(rep(0, times = T.mcDiff - lagTerms), 1:(Time - T.mcDiff - 1))} else{rep(0, times = Time-lagTerms-1)}
   tend.temp <- lagTerms:(Time-2)
-  Matrix::t(Matrix::bdiag(mapply(ti = ti.temp, t.end = tend.temp
+
+  if(inst.collapse){
+    Matrix::t(suppressWarnings(Matrix::Matrix(mapply(ti = ti.temp, t.end = tend.temp
+                                   , FUN = dat.fct, lagTerms = lagTerms, varname = varname
+                                   , MoreArgs = list(i = i, inst.collapse = inst.collapse
+                                                     #			, mc.ref.t = mc.ref.t
+                                                     , Time = Time, varname.i = varname.i, dat = dat, dat.na = dat.na)
+                                   , SIMPLIFY = TRUE), "dgCMatrix", sparse = TRUE)))
+  } else{
+    Matrix::t(Matrix::bdiag(mapply(ti = ti.temp, t.end = tend.temp
                                 , FUN = dat.fct, lagTerms = lagTerms, varname = varname
-                                , MoreArgs = list(i = i
+                                , MoreArgs = list(i = i, inst.collapse = inst.collapse
                                                  #			, mc.ref.t = mc.ref.t
                                                  , Time = Time, varname.i = varname.i, dat = dat, dat.na = dat.na)
                                 , SIMPLIFY = FALSE)))
+  }
   #   } else{
   #     t(sapply(X = (Time - T.mcDiff - 1):(Time - 2), FUN = dat.fct, i = i, varname = varname))
   #   }
@@ -81,6 +92,7 @@ variable.pre.fct	<- function(			# function that creates starting and end period
   ,lagTerms
   ,T.mcDiff
   ,i
+  ,inst.collapse
   #   ,mc.ref.t
   ,Time
   ,varname.i
@@ -92,10 +104,18 @@ variable.pre.fct	<- function(			# function that creates starting and end period
   ti.temp   <- rep(1, times = Time-lagTerms-1) + if(Time-lagTerms-1 - T.mcDiff > 0){c(rep(0, times = T.mcDiff - lagTerms), 1:(Time - T.mcDiff - 1))} else{rep(0, times = Time-lagTerms-1)}
   tend.temp <- (lagTerms+1):(Time-1)
 
-  Matrix::t(Matrix::bdiag(mapply(ti = ti.temp, t.end = tend.temp, FUN = dat.fct.pre, lagTerms = lagTerms, varname = varname
-                                   , MoreArgs = list(i = i, Time = Time
+  if(inst.collapse){
+    Matrix::t(suppressWarnings(Matrix::Matrix(mapply(ti = ti.temp, t.end = tend.temp, FUN = dat.fct.pre, lagTerms = lagTerms, varname = varname
+                                   , MoreArgs = list(i = i, inst.collapse = inst.collapse, Time = Time
+                                                     #			, mc.ref.t = mc.ref.t
+                                                     , varname.i = varname.i, dat = dat, dat.na = dat.na)
+                                    ), "dgCMatrix", sparse = TRUE)))
+  } else{
+    Matrix::t(Matrix::bdiag(mapply(ti = ti.temp, t.end = tend.temp, FUN = dat.fct.pre, lagTerms = lagTerms, varname = varname
+                                   , MoreArgs = list(i = i, inst.collapse = inst.collapse, Time = Time
                                                  #			, mc.ref.t = mc.ref.t
                                                  , varname.i = varname.i, dat = dat, dat.na = dat.na), SIMPLIFY = FALSE)))
+  }
   #   } else{
   #     t(sapply(X = (Time - T.mcDiff - 1):(Time - 1), FUN = dat.fct.pre, i = i, varname = varname))
   #   }
@@ -121,9 +141,10 @@ variable.pre.fct	<- function(			# function that creates starting and end period
 #'
 variable.ex.fct	<- function(			# function that creates starting and end period when deriving instruments from data for exogenous variables
   varname
-  ,lagTerms
   ,T.mcDiff
+  ,lagTerms
   ,i
+  ,inst.collapse
   #   ,mc.ref.t
   ,Time
   ,varname.i
@@ -135,24 +156,39 @@ variable.ex.fct	<- function(			# function that creates starting and end period w
 #  t.start		<- if(Time > T.mcDiff){ c((Time-T.mcDiff):(Time-lagTerms-1)) } else{ rep(1, times = Time-lagTerms-1) }
 #  t.start		<- if(Time > T.mcDiff){ c((Time-T.mcDiff):(Time)) } else{ rep(1, times = Time-lagTerms-1) }
 #  t.start   <- rep(1, times = Time-lagTerms-1) + if(Time-T.mcDiff > 0){c(rep(0, times = Time - T.mcDiff), (1:(Time - T.mcDiff)))} else{0}
-  if(inst.reg.ex.expand){
-    ti        <- rep(1, times = Time-lagTerms-1) + if(Time-T.mcDiff > 0){c(rep(0, times = Time-lagTerms-1-(Time-T.mcDiff)), (1:(Time - T.mcDiff)))} else{0}
-    t.end			<- ti + (T.mcDiff-1)
+  if(inst.collapse){
+    ti.temp   <- rep(1, times = Time-lagTerms-1) + if(Time-T.mcDiff > 0){c(rep(0, times = Time-lagTerms-1-(Time-T.mcDiff)), (1:(Time - T.mcDiff)))} else{0}
+    tend.temp <- (lagTerms+2):(Time)
     t.req.i   <- 1:(Time-lagTerms-1)
     t.req.e   <- (1:(Time-lagTerms-1)) + (lagTerms+1)
-  } else {
-    ti   <- rep(1, times = Time-lagTerms-1) + if(Time-T.mcDiff > 0){c(rep(0, times = Time-lagTerms-1-(Time-T.mcDiff)), (1:(Time - T.mcDiff)))} else{0}
-    t.end     <- (lagTerms+2):(Time)
-    t.req.i   <- 1:(Time-lagTerms-1)
-    t.req.e   <- (1:(Time-lagTerms-1)) + (lagTerms+1)
-  }
+
+    err.term.start	<- ti.temp-1
+    Matrix::t(suppressWarnings(Matrix::Matrix(mapply(ti = ti.temp, t.end = tend.temp, err.term.start = err.term.start, t.req.i = t.req.i, t.req.e = t.req.e, FUN = dat.fct.ex, varname = varname
+                                   , MoreArgs = list(i = i, inst.collapse = inst.collapse, Time = Time, lagTerms = lagTerms
+                                                     #				, mc.ref.t = mc.ref.t
+                                                     , varname.i = varname.i, dat = dat, dat.na = dat.na)
+                                    ), "dgCMatrix", sparse = TRUE)))
+
+  } else{
+    if(inst.reg.ex.expand){
+      ti.temp     <- rep(1, times = Time-lagTerms-1) + if(Time-T.mcDiff > 0){c(rep(0, times = Time-lagTerms-1-(Time-T.mcDiff)), (1:(Time - T.mcDiff)))} else{0}
+      tend.temp		<- ti + (T.mcDiff-1)
+      t.req.i     <- 1:(Time-lagTerms-1)
+      t.req.e     <- (1:(Time-lagTerms-1)) + (lagTerms+1)
+    } else {
+      ti.temp     <- rep(1, times = Time-lagTerms-1) + if(Time-T.mcDiff > 0){c(rep(0, times = Time-lagTerms-1-(Time-T.mcDiff)), (1:(Time - T.mcDiff)))} else{0}
+      tend.temp  <- (lagTerms+2):(Time)
+      t.req.i     <- 1:(Time-lagTerms-1)
+      t.req.e     <- (1:(Time-lagTerms-1)) + (lagTerms+1)
+    }
 #  t.end[t.end > Time]	<- Time
 #  err.term.start	<- c((min(t.start) + lagTerms + 1):max(t.end))
-  err.term.start	<- ti-1
-  Matrix::t(Matrix::bdiag(mapply(ti = ti, t.end = t.end, err.term.start = err.term.start, t.req.i = t.req.i, t.req.e = t.req.e, FUN = dat.fct.ex, varname = varname
-                                 , MoreArgs = list(i = i, Time = Time
+    err.term.start	<- ti.temp-1
+    Matrix::t(Matrix::bdiag(mapply(ti = ti.temp, t.end = tend.temp, err.term.start = err.term.start, t.req.i = t.req.i, t.req.e = t.req.e, FUN = dat.fct.ex, varname = varname
+                                 , MoreArgs = list(i = i, inst.collapse = inst.collapse, Time = Time
                                                  #				, mc.ref.t = mc.ref.t
                                                  , varname.i = varname.i, dat = dat, dat.na = dat.na), SIMPLIFY = FALSE)))																		# [M:] use all m.c. in direction of T and cut at initial periods
+  }
   #   } else{
   #    t(sapply(X = (Time - T.mcDiff - 1):(Time), FUN = dat.fct.ex, i = i, varname = varname, ...))
   #   }
@@ -188,6 +224,7 @@ dat.fct		<- function(			# function that creates instruments based on
   ,i												# renamed since 't()' is already a function
   ,lagTerms
   ,varname
+  ,inst.collapse
   ,Time
   #   ,mc.ref.t
   ,varname.i
@@ -195,11 +232,19 @@ dat.fct		<- function(			# function that creates instruments based on
   ,dat.na
 ){
   #   if(mc.ref.t){
-  dat[dat[, varname.i] == i, varname][ti:t.end]*					# if period t+1 and t+2 do not exist, t is not available as instrument
-    (as.numeric(!is.na(dat.na[dat.na[, varname.i] == i, varname][t.end-lagTerms+1] *
+  if(inst.collapse){
+    c(rev(dat[dat[, varname.i] == i, varname][ti:t.end]*					# if period t+1 and t+2 do not exist, t is not available as instrument
+      (as.numeric(!is.na(dat.na[dat.na[, varname.i] == i, varname][t.end-lagTerms+1] *
+                           dat.na[dat.na[, varname.i] == i, varname][t.end] *
+                           dat.na[dat.na[, varname.i] == i, varname][t.end+1] *
+                           dat.na[dat.na[, varname.i] == i, varname][t.end+2])))), rep(0, times = Time-length(ti:t.end)-lagTerms-1))
+  } else{
+    dat[dat[, varname.i] == i, varname][ti:t.end]*					# if period t+1 and t+2 do not exist, t is not available as instrument
+      (as.numeric(!is.na(dat.na[dat.na[, varname.i] == i, varname][t.end-lagTerms+1] *
                          dat.na[dat.na[, varname.i] == i, varname][t.end] *
                          dat.na[dat.na[, varname.i] == i, varname][t.end+1] *
                          dat.na[dat.na[, varname.i] == i, varname][t.end+2])))
+  }
   #   } else{
   #     dat[dat[, varname.i] == i, varname][ti]*						# if period T, T-1 and T-2 do not exist, t is not available as instrument
   #     (as.numeric(!is.na(dat.na[dat.na[, varname.i] == i, varname][Time] *
@@ -236,6 +281,7 @@ dat.fct.pre		<- function(
   ti
   ,t.end
   ,i												# renamed since 't' is already defined
+  ,inst.collapse
   ,lagTerms
   ,varname
   ,Time
@@ -245,10 +291,17 @@ dat.fct.pre		<- function(
   ,dat.na
 ){
   #   if(mc.ref.t){
-  dat[dat[, varname.i] == i, varname][ti:t.end]*
-    (as.numeric(!is.na(dat.na[dat.na[, varname.i] == i, varname][t.end-lagTerms+1] *
+  if(inst.collapse){
+    c(rev(dat[dat[, varname.i] == i, varname][ti:t.end]*
+      (as.numeric(!is.na(dat.na[dat.na[, varname.i] == i, varname][t.end-lagTerms+1] *
+                           dat.na[dat.na[, varname.i] == i, varname][t.end] *
+                           dat.na[dat.na[, varname.i] == i, varname][t.end+1])))), rep(0, times = Time-1-lagTerms+1-t.end))
+  } else{
+    dat[dat[, varname.i] == i, varname][ti:t.end]*
+      (as.numeric(!is.na(dat.na[dat.na[, varname.i] == i, varname][t.end-lagTerms+1] *
                          dat.na[dat.na[, varname.i] == i, varname][t.end] *
                          dat.na[dat.na[, varname.i] == i, varname][t.end+1])))
+  }
   #   } else{
   #     dat[dat[, varname.i] == i, varname][ti]*
   #     (as.numeric(!is.na(dat.na[dat.na[, varname.i] == i, varname][Time] *
@@ -282,8 +335,10 @@ dat.fct.ex		<- function(
   ,t.end												# renamed since 't' is already defined
   ,t.req.i
   ,t.req.e
+  ,lagTerms
   ,err.term.start
   ,i
+  ,inst.collapse
   ,varname
   ,Time
   #   ,mc.ref.t
@@ -292,11 +347,21 @@ dat.fct.ex		<- function(
   ,dat.na
 ){
   #   if(mc.ref.t){
-  dat[dat[, varname.i] == i, varname][ti:t.end]*
-    (as.numeric(!is.na(dat.na[dat.na[, varname.i] == i, varname][ti:t.end] *
-                         dat.na[dat.na[, varname.i] == i, varname][rep((err.term.start+2), times = length(ti:t.end))])))*
-    as.numeric(!is.na(dat.na[dat.na[, varname.i] == i, varname][rep(t.req.i, times = length(ti:t.end))]))*
-    as.numeric(!is.na(dat.na[dat.na[, varname.i] == i, varname][rep(t.req.e, times = length(ti:t.end))]))
+  if(inst.collapse){
+    c(rev(dat[dat[, varname.i] == i, varname][ti:t.end]*
+      (as.numeric(!is.na(dat.na[dat.na[, varname.i] == i, varname][ti:t.end])))), rep(0, times = Time - length(ti:t.end)))
+#                           *dat.na[dat.na[, varname.i] == i, varname][rep((err.term.start+2), times = length(ti:t.end))])))*
+#        as.numeric(!is.na(dat.na[dat.na[, varname.i] == i, varname][rep(t.req.i, times = Time)]))*
+#        as.numeric(!is.na(dat.na[dat.na[, varname.i] == i, varname][rep(t.req.e, times = Time)]))), rep(0, times = Time-1-lagTerms))
+#      as.numeric(!is.na(dat.na[dat.na[, varname.i] == i, varname][rep(t.req.i, times = length(ti:t.end))]))*
+#      as.numeric(!is.na(dat.na[dat.na[, varname.i] == i, varname][rep(t.req.e, times = length(ti:t.end))]))), rep(0, times = Time-1-lagTerms))
+  } else{
+    dat[dat[, varname.i] == i, varname][ti:t.end]*
+      (as.numeric(!is.na(dat.na[dat.na[, varname.i] == i, varname][ti:t.end])))
+#                         *dat.na[dat.na[, varname.i] == i, varname][rep((err.term.start+2), times = length(ti:t.end))])))*
+#      as.numeric(!is.na(dat.na[dat.na[, varname.i] == i, varname][rep(t.req.i, times = length(ti:t.end))]))*
+#      as.numeric(!is.na(dat.na[dat.na[, varname.i] == i, varname][rep(t.req.e, times = length(ti:t.end))]))
+  }
 #    (as.numeric(!is.na(dat.na[dat.na[, varname.i] == i, varname][ti:t.end] *
 #                        dat.na[dat.na[, varname.i] == i, varname][rep((err.term.start+2), times = length(ti:t.end))])))
   #                         dat.na[dat.na[, varname.i] == i, varname][rep((err.term.start-2), times = length(ti:t.end))])))
@@ -350,6 +415,7 @@ dat.fct.ex		<- function(
 LEV.fct	<- function(
   varname
   ,i
+  ,inst.collapse
   ,T.mcLev
   ,lagTerms
   ,use.mc.diff
@@ -366,9 +432,15 @@ LEV.fct	<- function(
     ti.temp   <- max(2,lagTerms)
     tend.temp <- Time-1
 
-    Matrix::Diagonal(do.call(what = datLEV.fct, args = list(ti = ti.temp, t.end = tend.temp, i = i, varname = varname, lagTerms = lagTerms, use.mc.diff = use.mc.diff, inst.stata = inst.stata
+    if(inst.collapse){
+      Matrix::Matrix(do.call(what = datLEV.fct, args = list(ti = ti.temp, t.end = tend.temp, i = i, varname = varname, lagTerms = lagTerms, use.mc.diff = use.mc.diff, inst.stata = inst.stata
                                                             #				, mc.ref.t = mc.ref.t
-                                                            , dat.na = dat.na, dat = dat, varname.i = varname.i, Time = Time)), n = Time-max(2,lagTerms))
+                                                            , dat.na = dat.na, dat = dat, varname.i = varname.i, Time = Time)))
+    } else{
+      Matrix::Diagonal(do.call(what = datLEV.fct, args = list(ti = ti.temp, t.end = tend.temp, i = i, varname = varname, lagTerms = lagTerms, use.mc.diff = use.mc.diff, inst.stata = inst.stata
+                                                              #				, mc.ref.t = mc.ref.t
+                                                              , dat.na = dat.na, dat = dat, varname.i = varname.i, Time = Time)), n = Time-max(2,lagTerms))
+    }
     #     }
   } else{
     #     if(mc.ref.t){
@@ -380,6 +452,9 @@ LEV.fct	<- function(
                                                    #					, mc.ref.t = mc.ref.t
                                                    , dat.na = dat.na, dat = dat, varname.i = varname.i, Time = Time)) ))*
       as.vector(!is.na(diff(dat.na[dat.na[, varname.i] == i, varname][(max(2,lagTerms)-1):(Time-1)])))
+
+    #note: function not adjusted for 'collapsing' when no moment conditions from equations in differences are present
+    #
     #     } else{
     #       t(mapply(ti = Time - T.mcLev, t.end = Time - 1, FUN = datLEV.fct, i = i, varname = varname,
     #		MoreArgs = list(use.mc.diff = use.mc.diff, inst.stata = inst.stata, mc.ref.t = mc.ref.t, dat.na = dat.na, dat = dat, varname.i = varname.i, Time = Time)))
@@ -425,6 +500,7 @@ LEV.fct	<- function(
 LEV.pre.fct	<- function(
   varname
   ,i
+  ,inst.collapse
   ,T.mcLev
   ,lagTerms
   ,use.mc.diff
@@ -440,9 +516,15 @@ LEV.pre.fct	<- function(
     ti.temp   <- max(2,lagTerms)
     tend.temp <- Time
 
-    Matrix::Diagonal(do.call(what = datLEV.pre.fct, args = list(ti = ti.temp, t.end = tend.temp, lagTerms = lagTerms, varname = varname, i = i, use.mc.diff = use.mc.diff, inst.stata = inst.stata
+    if(inst.collapse){
+      Matrix::Matrix(do.call(what = datLEV.pre.fct, args = list(ti = ti.temp, t.end = tend.temp, lagTerms = lagTerms, varname = varname, i = i, use.mc.diff = use.mc.diff, inst.stata = inst.stata
+                                                                  #			, mc.ref.t = mc.ref.t
+                                                                  , dat = dat, dat.na = dat.na, varname.i = varname.i, Time = Time)))
+    } else{
+      Matrix::Diagonal(do.call(what = datLEV.pre.fct, args = list(ti = ti.temp, t.end = tend.temp, lagTerms = lagTerms, varname = varname, i = i, use.mc.diff = use.mc.diff, inst.stata = inst.stata
                                                                 #			, mc.ref.t = mc.ref.t
                                                                 , dat = dat, dat.na = dat.na, varname.i = varname.i, Time = Time)), n = Time-max(2,lagTerms)+1)
+    }
     #     }
   } else{
     #     if(mc.ref.t){
@@ -455,6 +537,8 @@ LEV.pre.fct	<- function(
                                                    , dat = dat, dat.na = dat.na, varname.i = varname.i, Time = Time)) *
       as.vector(!is.na(diff(dat.na[dat.na[, varname.i] == i, varname][(lagTerms-1):Time]))) ) ) )
 
+    #note: function not adjusted for 'collapsing' when no moment conditions from equations in differences are present
+    #
     #     } else{
     #       t(mapply(ti = Time - T.mcLev, t.end = Time, FUN = datLEV.pre.fct, varname = varname,
     #		MoreArgs = list(i = i, use.mc.diff = use.mc.diff, inst.stata = inst.stata
@@ -583,10 +667,10 @@ datLEV.pre.fct		<- function(
   ,dat.na
 ){
   #   if(mc.ref.t){
-  if(is.na(dat.na[dat.na[, varname.i] == i, varname][ti])){
-    ti	= ti+1
-    t.end	= t.end+1
-  }
+#  if(is.na(dat.na[dat.na[, varname.i] == i, varname][ti])){
+#    ti	= ti+1
+#    t.end	= t.end+1
+#  }
 
   if(use.mc.diff & !(inst.stata)){
 
@@ -682,6 +766,7 @@ Z_i.fct	<- function(
   ,use.mc.nonlinAS
   ,include.y
   ,varname.y
+  ,inst.collapse
   ,inst.stata
   ,include.dum
   ,dum.diff
@@ -715,48 +800,54 @@ Z_i.fct	<- function(
   if(use.mc.diff){
     #     if(mc.ref.t){
     if(include.y){
-      Z_i.mc.diff_end.y	<- do.call(what = "cbind", args = sapply(X = varname.y, FUN = variable.fct, i = i, T.mcDiff = maxLags.y,
-                                                                 lagTerms = max.lagTerms
+      Z_i.mc.diff_end.y	<- do.call(what = "cbind", args = sapply(X = varname.y, FUN = variable.fct, i = i,
+                                                                 inst.collapse = inst.collapse,
+                                                                 T.mcDiff = maxLags.y, lagTerms = max.lagTerms
                                                                  #						, mc.ref.t = mc.ref.t
                                                                  , Time = Time, varname.i = varname.i, dat = dat, dat.na = dat.na) )
     }
     if(include.x){
       if(end.reg){
         if(length(varname.reg.end) == 1){
-          Z_i.mc.diff_end.x	<- do.call(what = "cbind", args = sapply(FUN = variable.fct, varname.reg.end, i = i, T.mcDiff = maxLags.reg.end,
-                                                                     lagTerms = max.lagTerms
+          Z_i.mc.diff_end.x	<- do.call(what = "cbind", args = sapply(FUN = variable.fct, varname.reg.end, i = i,
+                                                                     inst.collapse = inst.collapse,
+                                                                     T.mcDiff = maxLags.reg.end, lagTerms = max.lagTerms
                                                                      #						, mc.ref.t = mc.ref.t
                                                                      , Time = Time, varname.i = varname.i, dat = dat, dat.na = dat.na) )
         } else{
           Z_i.mc.diff_end.x	<- do.call(what = "cbind", args = mapply(FUN = variable.fct, varname.reg.end, T.mcDiff = maxLags.reg.end
                                                                    #						, mc.ref.t = mc.ref.t
-                                                                   , MoreArgs = list(i = i, Time = Time, varname.i = varname.i, lagTerms = max.lagTerms
+                                                                   , MoreArgs = list(i = i, inst.collapse = inst.collapse
+                                                                                     , Time = Time, varname.i = varname.i, lagTerms = max.lagTerms
                                                                                      , dat = dat, dat.na = dat.na)) )
         }
       }
       if(pre.reg){
         if(length(varname.reg.pre) == 1){
-          Z_i.mc.diff_pre	<- do.call(what = "cbind", args = sapply(FUN = variable.pre.fct, varname.reg.pre, i = i, T.mcDiff = maxLags.reg.pre,
-                                                                   lagTerms = max.lagTerms
+          Z_i.mc.diff_pre	<- do.call(what = "cbind", args = sapply(FUN = variable.pre.fct, varname.reg.pre, i = i, inst.collapse = inst.collapse,
+                                                                   T.mcDiff = maxLags.reg.pre, lagTerms = max.lagTerms
                                                                    #						, mc.ref.t = mc.ref.t
                                                                    , Time = Time, varname.i = varname.i, dat = dat, dat.na = dat.na) )
         } else{
           Z_i.mc.diff_pre	<- do.call(what = "cbind", args = mapply(FUN = variable.pre.fct, varname.reg.pre, T.mcDiff = maxLags.reg.pre
                                                                  #						, mc.ref.t = mc.ref.t
-                                                                 , MoreArgs = list(i = i, Time = Time, varname.i = varname.i, lagTerms = max.lagTerms
+                                                                 , MoreArgs = list(i = i, inst.collapse = inst.collapse
+                                                                                   , Time = Time, varname.i = varname.i, lagTerms = max.lagTerms
                                                                                    , dat = dat, dat.na = dat.na)) )
         }
       }
       if(ex.reg){
         if(length(varname.reg.ex) == 1){
-          Z_i.mc.diff_ex	<- do.call(what = "cbind", args = sapply(FUN = variable.ex.fct, varname.reg.ex, i = i, T.mcDiff = maxLags.reg.ex,
+          Z_i.mc.diff_ex	<- do.call(what = "cbind", args = sapply(FUN = variable.ex.fct, varname.reg.ex, i = i, inst.collapse = inst.collapse,
+                                                                  T.mcDiff = maxLags.reg.ex,
                                                                   lagTerms = max.lagTerms, inst.reg.ex.expand = inst.reg.ex.expand
                                                                   #						, mc.ref.t = mc.ref.t
                                                                   , Time = Time, varname.i = varname.i, dat = dat, dat.na = dat.na) )
         } else{
           Z_i.mc.diff_ex	<- do.call(what = "cbind", args = mapply(FUN = variable.ex.fct, varname.reg.ex, T.mcDiff = maxLags.reg.ex
                                                                 #						, mc.ref.t = mc.ref.t
-                                                                , MoreArgs = list(i = i, Time = Time, varname.i = varname.i, lagTerms = max.lagTerms
+                                                                , MoreArgs = list(i = i, inst.collapse = inst.collapse
+                                                                                  , Time = Time, varname.i = varname.i, lagTerms = max.lagTerms
                                                                                   , inst.reg.ex.expand = inst.reg.ex.expand
                                                                                   , dat = dat, dat.na = dat.na)) )
         }
@@ -773,23 +864,26 @@ Z_i.fct	<- function(
   if(use.mc.lev){
     #     if(mc.ref.t){
     if(include.y){
-      Z_i.mc.lev_end.y	<- do.call(what = "cbind", args = sapply(X = varname.y, FUN = LEV.fct, i = i, T.mcLev = maxLags.y, lagTerms = max.lagTerms,
+      Z_i.mc.lev_end.y	<- do.call(what = "cbind", args = sapply(X = varname.y, FUN = LEV.fct, i = i, inst.collapse = inst.collapse,
+                                                                T.mcLev = maxLags.y, lagTerms = max.lagTerms,
                                                                 use.mc.diff = use.mc.diff, inst.stata = inst.stata
                                                                 #						, mc.ref.t = mc.ref.t
-                                                                , Time = Time, varname.i = varname.i, dat = dat, dat.na = dat.na) )
+                                                                , Time = Time, varname.i = varname.i, dat = dat, dat.na = dat.na, simplify = FALSE) )
     }
     if(include.x){
       if(end.reg){
         if(length(varname.reg.end) == 1){
-          Z_i.mc.lev_end.x	<- do.call(what = "cbind", args = sapply(FUN = LEV.fct, i = i, varname.reg.end, T.mcLev = maxLags.reg.end, lagTerms = max.lagTerms,
+          Z_i.mc.lev_end.x	<- do.call(what = "cbind", args = sapply(FUN = LEV.fct, i = i, inst.collapse = inst.collapse,
+                                                                    varname.reg.end, T.mcLev = maxLags.reg.end, lagTerms = max.lagTerms,
                                                                     use.mc.diff = use.mc.diff, inst.stata = inst.stata
                                                                     #						, mc.ref.t = mc.ref.t
-                                                                    , Time = Time, varname.i = varname.i, dat = dat, dat.na = dat.na) )
+                                                                    , Time = Time, varname.i = varname.i, dat = dat, dat.na = dat.na, simplify = FALSE) )
         } else{
           Z_i.mc.lev_end.x	<- do.call(what = "cbind", args = mapply(FUN = LEV.fct, varname.reg.end, T.mcLev = maxLags.reg.end
                                                                   , MoreArgs = list(use.mc.diff = use.mc.diff, inst.stata = inst.stata
                                                                   #						, mc.ref.t = mc.ref.t
-                                                                        , i = i, Time = Time, varname.i = varname.i, lagTerms = max.lagTerms
+                                                                        , i = i, inst.collapse = inst.collapse
+                                                                        , Time = Time, varname.i = varname.i, lagTerms = max.lagTerms
                                                                         , dat = dat, dat.na = dat.na)) )
         }
       }
@@ -797,17 +891,19 @@ Z_i.fct	<- function(
         varname.ex.pre.temp <- c({if(!(is.null("varname.reg.ex"))){varname.reg.ex}}, {if(!(is.null("varname.reg.pre"))){varname.reg.pre}} )
         T.mcLev.temp    <- c({if(!(is.null("varname.reg.ex"))){maxLags.reg.ex - 1}}, {if(!(is.null("varname.reg.pre"))){maxLags.reg.pre}} )
         if(length(varname.ex.pre.temp) == 1){
-          Z_i.mc.lev_ex.pre	<- do.call(what = "cbind", args = sapply(FUN = LEV.pre.fct, i = i, varname.ex.pre.temp, T.mcLev = T.mcLev.temp
+          Z_i.mc.lev_ex.pre	<- do.call(what = "cbind", args = sapply(FUN = LEV.pre.fct, i = i, inst.collapse = inst.collapse
+                                                                    , varname.ex.pre.temp, T.mcLev = T.mcLev.temp
                                                                     , use.mc.diff = use.mc.diff, inst.stata = inst.stata
                                                                      #						,mc.ref.t = mc.ref.t
                                                                     , Time = Time, varname.i = varname.i, lagTerms = max.lagTerms
-                                                                    , dat = dat, dat.na = dat.na) )
+                                                                    , dat = dat, dat.na = dat.na, simplify = FALSE) )
         } else{
           Z_i.mc.lev_ex.pre	<- do.call(what = "cbind", args = mapply(FUN = LEV.pre.fct, varname.ex.pre.temp, T.mcLev = T.mcLev.temp
-                                                                     ,MoreArgs = list(i = i, use.mc.diff = use.mc.diff, inst.stata = inst.stata
+                                                                     ,MoreArgs = list(i = i, inst.collapse = inst.collapse
+                                                                        , use.mc.diff = use.mc.diff, inst.stata = inst.stata
                                                                                       #						,mc.ref.t = mc.ref.t
                                                                         , Time = Time, varname.i = varname.i, lagTerms = max.lagTerms
-                                                                        , dat = dat, dat.na = dat.na)) )
+                                                                        , dat = dat, dat.na = dat.na), SIMPLIFY = FALSE) )
         }
       }
 
