@@ -18,9 +18,11 @@ varname.i = "firm"
 varname.t = "year"
 
 use.mc.diff = TRUE
-use.mc.lev = FALSE
+use.mc.lev = TRUE
 use.mc.nonlin = FALSE
 use.mc.nonlinAS = NULL
+#inst.collapse = FALSE
+inst.collapse = TRUE
 inst.stata = FALSE
 
 include.y = TRUE
@@ -29,14 +31,26 @@ lagTerms.y = 1
 maxLags.y = NULL
 
 include.x = TRUE
-varname.reg.end = c("wage","capital")
-lagTerms.reg.end = c(0,0)
-maxLags.reg.end = c(3,3)
+#varname.reg.end = c("wage")
+#lagTerms.reg.end = c(0)
+#maxLags.reg.end = c(3)
+#varname.reg.end = c("wage","capital")
+#lagTerms.reg.end = c(0,0)
+varname.reg.end = NULL
+lagTerms.reg.end = NULL
+#maxLags.reg.end = c(3,3)
+maxLags.reg.end = NULL
+#varname.reg.pre = c("wage","capital")
+#lagTerms.reg.pre = c(0,0)
 varname.reg.pre = NULL
 lagTerms.reg.pre = NULL
 maxLags.reg.pre = NULL
-varname.reg.ex = NULL
-lagTerms.reg.ex = NULL
+#varname.reg.ex = "wage"
+lagTerms.reg.ex = c(0,0)
+varname.reg.ex = c("wage","capital")
+maxLags.reg.ex = c(3,3)
+#varname.reg.ex = NULL
+#lagTerms.reg.ex = NULL
 maxLags.reg.ex = NULL
 include.x.instr = FALSE
 varname.reg.instr = NULL
